<?php

namespace Drupal\md_wordscloud;

use Drupal\Core\Database\Connection;

/**
 * Service that handles wordcloud data.
 */
class WordcloudsContent {

  /**
   * Database Connection Service.
   *
   * @var Drupal\Core\Database\Connection
   */
  protected $dbConn;

  /**
   * Constructor.
   */
  public function __construct(
        Connection $db_connection
    ) {
    $this->dbConn = $db_connection;
  }

  /**
   * Get keyword list.
   *
   * @return array
   *   keyword List.
   */
  public function getCloudKeywordList($vids, $size) {
    $cloud_list = $this->getmdcloudTags($vids, $size);
    $terms_url = $words = $counts = [];
    foreach ($cloud_list as $row) {
      $counts[] = $row->count;
      $words[] = $row->name;
      $terms_url[$row->name] = '#';
    }
    $result = (object) [
      'terms_url' => $terms_url,
      'words' => $words,
      'counts' => $counts,
    ];

    return $result;
  }

  /**
   * {@inheritdoc}
   */
  public function getmdcloudTags(array $vids, $size) {
    $tags = [];
    $query = $this->dbConn->select('taxonomy_term_data', 'td');
    $query->addExpression('COUNT(td.tid)', 'count');
    $query->fields('tfd', ['name', 'description__value']);
    $query->fields('td', ['tid', 'vid']);
    $query->addExpression('MIN(tn.nid)', 'nid');

    $query->join('taxonomy_index', 'tn', 'td.tid = tn.tid');
    $query->join('node_field_data', 'n', 'tn.nid = n.nid');
    $query->join('taxonomy_term_field_data', 'tfd', 'tfd.tid = tn.tid');

    $query->condition('td.vid', $vids, 'IN');
    $query->condition('n.status', 1);
    $query->condition('tfd.status', 1);

    $query->groupBy('td.tid')->groupBy('td.vid')->groupBy('tfd.name');
    $query->groupBy('tfd.description__value');

    $query->having('COUNT(td.tid)>0');
    $query->orderBy('count', 'DESC');

    if ($size > 0) {
      $query->range(0, $size);
    }
    $result = $query->execute()->fetchAll();

    foreach ($result as $tag) {
      $tags[$tag->tid] = $tag;
    }

    return $tags;
  }

}
